<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
//
// +----------------------------------------------------------------------+
// | PHP Version 4                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2002 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.02 of the PHP license,      |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/2_02.txt.                                 |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Alexander Zhukov <alex@veresk.ru> Original port from Python |
// | Authors: Harry Fuecks <hfuecks@phppatterns.com> Port to PEAR + more  |
// +----------------------------------------------------------------------+
//
// $Id$
//
/**
 * Handles parsing of element attributes
 * @author Alexander Zhukov <alex@veresk.ru>
 * @author Harry Fuecks <hfuecks@phppatterns.com>
 * @access protected
 * @package XML_HTMLSax
 */

class Attributes_Parser
{
    /**
     * Instance of State_Machine
     * @see State_Machine
     * @var object
     * @access private
     */
    var $sm;

    /**
     * Incoming string containing attributes to parse
     * @var string
     * @access private
     */
    var $str;

    /**
     * Array of parsed attributes to be returned
     * @var array 
     * @access private
     */
    var $atts = array();

    /**
     * Temporary store for attribute name
     * @var string
     * @access private
     */
    var $_tmp_attname = '';

    /**
     * Temporary store for attribute value
     * @var string
     * @access private
     */
    var $_tmp_value = '';

    /**
     * Last attribute parsed
     * @var string
     * @access private
     */
    var $_last_att = '';
    
    /**
     * Constructs Attributes_Parser
     */
    function Attributes_Parser()
    {
        $this->sm = new State_Machine();
    }

    /**
     * Invokes parsing of attributes
     *
     * @param string to parse
     * @return array of attributes
     * @access protected
     */
    function parse($str)
    {
        $this->str = $html = preg_replace('/\s*=\s*/', '=',trim($str)).' ';
        $this->sm->att_name_start = true;
        $this->_parse();
        return $this->atts;
    }

    /**
     * Parses the attributes, maintaining the state machine
     *
     * @return void
     * @access private
     */
    function _parse()
    {
        for ($i=0; $i < strlen($this->str); $i++)
        {
            $chr  = $this->str[$i];
            if ($this->sm->att_val_start and ($chr == "\"" || $chr == "'"))
            {
                if($this->sm->quote)
                {
                    $this->sm->quote = false;
                }
                else 
                {
                    $this->sm->quote = true;
                }
            }
            
            if (($chr == '=') and ($this->sm->quote == false))
            { 
                $this->sm->att_name_start = false;
                $this->sm->att_name_end = true;
            }
            if (($chr == ' ') and ($this->sm->quote == false))
            {
                $this->sm->att_val_start = false;
                $this->sm->att_val_end = true;
            }

            $this->_doChar($chr);
        }
    }
    
     /**
     * Parses a single character
     *
     * @return void
     * @access private
     */
    function _doChar($chr)
    {
        if ($this->sm->att_name_start) { $this->_tmp_attname .= $chr; }
        if ($this->sm->att_name_end) 
        { 
            $this->_last_att = trim($this->_tmp_attname);
            $this->atts[trim($this->_tmp_attname)] = $this->_last_att;
            $this->_tmp_attname = '';
            $this->sm->att_name_end = false;
            $this->sm->att_val_start = true;
        }
        if ($this->sm->att_val_start) { $this->_tmp_value .= $chr; }
        if ($this->sm->att_val_end)
        {
            if ($this->_tmp_value != '') 
            { 
                $this->_tmp_value = substr($this->_tmp_value,1); 
                if (in_array (substr($this->_tmp_value,0,1), array ("\"", "'")))
                {
                    $this->_tmp_value = substr($this->_tmp_value,1); 
                }
                if (in_array (substr($this->_tmp_value,-1), array ("\"", "'")))
                {
                    $this->_tmp_value = substr($this->_tmp_value,0,-1); 
                }
                $this->atts[$this->_last_att] = $this->_tmp_value;
            }
            $this->_tmp_value = '';
            $this->sm->att_val_end = false;
            if ( !$this->sm->att_name_start )
            {
                $this->sm->att_name_start = true;
            }
            else
            {
                $this->atts[trim($this->_tmp_attname)] = true;
                $this->_tmp_attname = '';
            }
        }
    }
}
?>